@@ -1,7 +1,7 @@
 /*
  * This file is part of budgie-desktop
  *
- * Copyright (C) 2015-2016 Ikey Doherty <ikey@solus-project.com>
+ * Copyright © 2015-2019 Budgie Desktop Developers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,14 +15,11 @@
 
 gchar *budgie_form_theme_path(const gchar *suffix)
 {
-        gint minor_version = gtk_get_minor_version();
+        guint minor_version = gtk_get_minor_version();
 
-        /* Prioritize 3.18 */
         switch (minor_version) {
-        case 18:
-                return g_strdup_printf("%s/3.18/%s", THEME_PREFIX, suffix);
         case 20:
         default:
-                return g_strdup_printf("%s/3.20/%s", THEME_PREFIX, suffix);
+                return g_strdup_printf("%s/%s_3.20.css", THEME_PREFIX, suffix);
         }
 }
